#define USAGE "dra [-Reflectance] [-h] [-L <lutfile>] [-o <outname>] rootname"

#include <stdio.h>
#include "dralib.h"
#define uchar unsigned char
#define ulong unsigned long
extern char progname[31];
char *outname =  "dra";
char *rootname =  NULL;

#define MAX_IMAGE XSIZE*YSIZE
float f_image[MAX_IMAGE];
uchar image[MAX_IMAGE];
char debugging=0;
char register_flag=0;
char old_590 = 0;
char correcting=1;
char oa_flag=1;
float waveband_trans[NB_FILTERS]; 
char *lut_filename = LUT_FILENAME;
float *off_axis_image=NULL;
char refl_flag=0;
char hdr_flag=1;

main (argc, argv)
int argc;
char *argv[];
{
  sprintf( progname, "dra" );
  while( (--argc>0) && ((*++argv)[0] == '-') )
    switch( (*argv)[1] ) 
    {
      case 'L' : if( !argc-->0 ) error( USAGE );
		 lut_filename = *++argv;
		 break;

      case 'o' : if( !argc-->0 ) error( USAGE );
		 outname = *++argv;
		 break;

      case 'R' : refl_flag=!refl_flag; break;
      case '5' : old_590=!old_590; break;
      case 'r' : register_flag=!register_flag; break;
      case 'C' : correcting=!correcting; break;
      case 'O' : oa_flag=!oa_flag; break;
      case 'h' : hdr_flag=!hdr_flag; break;
      case 'd' : debugging=!debugging; break;
      default : error( USAGE ); 
  }
  if( !argc-->0 ) error( USAGE );
  rootname=*argv++;
  if( oa_flag ) off_axis_image = mk_off_axis_image();
  /* if( oa_flag )
  write_image( OFF_AXIS_FILENAME, off_axis_image, (ulong) 0,IMP3,XSIZE,YSIZE);
  */
  if( oa_flag ) invert_image( off_axis_image, XSIZE*YSIZE );
  read_lut_image( lut_filename );
  process_files( rootname, outname );
}

invert_image( image, n )
register float *image;
register ulong n;
{
  for( ; n-->0; image++ ) *image = 1.0/(*image);
}

compute_waveband_trans( old_590 )
char old_590;
{
 int i;
  if( old_590 ) use_old_590_filter();
  for( i=0; i<NB_FILTERS; i++ )
  {
    waveband_trans[i] = filter_data[i].trans * filter_data[i].tube_sensi;
    if( correcting ) waveband_trans[i] /= filter_data[i].corr_factor;
  }
}

process_files( rootname, outname )
char *rootname, *outname;
{
 ulong userdata;
 short type, xs, ys;
 char in_file[256];
 char out_file[256];
 FILE *fpin=NULL;
 int grey_x, grey_y;
 int i, it, aper_index;
  set_dra_flags( rootname, &register_flag, &old_590, &grey_x, &grey_y );
  fprintf( stderr,"%s: register_flag=%d old_590=%d grey=(%d %d)\n", 
             rootname, register_flag,   old_590, grey_x, grey_y );
  compute_waveband_trans( old_590 );
  for( i=0; i<NB_FILTERS; i++ )
  {
    sprintf( in_file,  "%s.%3d", rootname, filter_data[i].label );
    sprintf( out_file, "%s.%3d", outname,  filter_data[i].label );

    fprintf(stderr, "processing file: %s ...\n", in_file );
    if( NULL == (fpin = fopen( in_file, "r" ))) error("can't read file"); 
    readTdata_imp_hdr( fileno(fpin), &type, &userdata, &xs, &ys );
    filter_data[i].it = it = IT( userdata ); aper_index = AP( userdata );
    if( it<1 || LUT_XS<it ) error( "bad it in image header" );
    if( it == LUT_XS ) it--; /* hack */
    if( aper_index<1 || MAX_AP<=aper_index )
      error("bad aperture id in image_header");

    if( type != IMP1 ) error( "input image not IMP1"  );
    if( xs!=XSIZE || ys!=YSIZE )  error("input image not 256x256");
    read_bytes( fileno(fpin), image, xs*ys );
    fclose( fpin );

    if( register_flag ) register_image( image,
      filter_data[i].reg_x, filter_data[i].reg_y,  xs, ys );
    irradiance( f_image, image, it, xs, ys );
    scene_radiance( f_image, waveband_trans[i], aper[aper_index].trans, xs*ys );
    if( refl_flag )
    {
      if(!reflectance( f_image, xs, ys, grey_x, grey_y, KODAK_GREY ))
	error("couldn't derive reflectance for image set");
    }
    write_image( out_file, f_image, userdata, IMP3, xs, ys );
  }
}

reflectance( f_image, xs, ys, grey_x, grey_y, card_refl )
float *f_image;
int xs, ys, grey_x, grey_y;
float card_refl; /* in range [0,1] */
{
 double card_rad, f;
 int i,j;
  if( grey_x<1 || xs-1<=grey_x ) return(0);
  if( grey_y<1 || ys-1<=grey_y ) return(0);
  for( card_rad=0.0, j= -1; j<=1; j++ )  /* 3x3 local average */
    for( i= -1; i<=1; i++ )
      card_rad += f_image[ (grey_x+i) + xs*(grey_y+j)];
  card_rad /= 9.0;
  if( card_rad <= 0.0 ) return(0);
  f = card_refl/card_rad;
  for( i=0; i<xs*ys; i++ ) *f_image++ *= f;
  return(1);
}

write_image( out_file, image, userdata, type, xs, ys )
char *out_file;
uchar *image;
ulong userdata;
short type, xs, ys;
{
 FILE *fpout=NULL;
 int unit;
  if( NULL == (fpout = fopen( out_file, "w" ))) error("can't write file"); 
  if( hdr_flag )
    writeTdata_imp_hdr( fileno(fpout), type, userdata, xs, ys );
  switch(type)
  {
    case IMP1 : unit=sizeof(uchar); break;
    case IMP3 : unit=sizeof(float); break;
    default   : error("impossible case in write_image");
  }
  write_bytes( fileno(fpout), image, xs*ys*unit );
  fclose( fpout );
}

scene_radiance( f_image, t, a, n )
float *f_image;
float t, a;
ulong n;
{
 float factor= 1.0/(a*t);
  for( ; n-->0; f_image++ ) *f_image *= factor;
}

irradiance( f_image, image, it, xs, ys )
float *f_image;
uchar *image;
int it;
short xs,ys;
{
 ulong offs;
 int i,j;
  if( it<1 || LUT_XS<it ) error("impossible case in irradiance()");
  for( j=0,offs=0; j<ys; j++ )
    for( i=0; i<xs; i++,offs++ )
    {
      f_image[offs] = get_lut( image[offs], it );
      if( oa_flag ) f_image[offs] *= off_axis_image[offs];
    }
}

cp_buf( out, in, n )
register uchar *out, *in;
register ulong n;
{ for( ; n-->0; ) *out++ = *in++; }

register_image( in, fix_x, fix_y, xs, ys )
uchar *in;
int fix_x, fix_y;
short xs, ys;
{
 uchar out[MAX_IMAGE];
 int j,jj;
  if( fix_x == 0 && fix_y == 0 ) return(1);
  for( j=0; j<ys; j++ )
  {
    jj= j-fix_y;
    if(  jj<0  ) jj=0;
    if( ys<=jj ) jj=ys-1;
    register_line( &(out[j*xs]), &(in[jj*xs]), fix_x, xs );
  }
  cp_buf( in, out, xs*ys );
  return(1);
}

register_line( out, in, fix_x, xs )
uchar *out, *in;
int fix_x;
short xs;
{
 int i,ii;
  for(i=0; i<xs; i++ )
  {
    ii= i-fix_x;
    if(  ii<0  ) ii=0;
    if( xs<=ii ) ii=xs-1;
    out[i] = in[ii];
  }
}
