/* dralib.c */
#include <stdio.h>
#include "dralib.h"
#define ulong unsigned long
#define uchar unsigned char

static float off_axis_coefs[5] = {
  /*  1       x           y          root_x    root_y  */
  1.14792, -0.00103170, -0.00138109, 0.00796543, 0.0082625
  /* 1.09567, -0.00125809, -0.00162050, 0.01299200, 0.0132874 */
};

init_root( root, n )
double *root;
int n;
{
 double sqrt();
 int i;
  for( i=0; i<n; i++ )
    root[i] = sqrt((double) i);
  return(1);
}

float off_axis( i, j )  /* one-off look up */
int i,j;
{
 static double root[XSIZE];
 static done=0;
 float val;
  if( !done ) done = init_root( root, XSIZE );
  val = off_axis_coefs[0] + off_axis_coefs[1]*i + off_axis_coefs[2]*j +
        off_axis_coefs[3]*root[i] + off_axis_coefs[4]*root[j];
  return(val); 
}


float *mk_off_axis_image()
{
 float *im=NULL, off_axis();
 unsigned long offs;
 int i,j;
  if( NULL == (im = (float *) malloc( XSIZE*YSIZE*sizeof(float) )))
    error("can't allocate space in mk_off_axis_image");
  for( j=0,offs=0; j<YSIZE; j++ )
    for( i=0; i<XSIZE; i++,offs++ )
       im[offs] = off_axis( i, j );
  return( im );
}

use_old_590_filter()
{
  filter_data[19].trans=6.034366;
  filter_data[19].reg_x=0;
  filter_data[19].reg_y=0;
}

APER aper[MAX_AP] = { { "      ", 0.0 },
 { "f16", 1.0/256 }, { "f11", 1.0/128 }, {  "f8", 1.0/64  }, {"f5.6", 1.0/32  },
 {  "f4", 1.0/16  }, {"f2.8", 1.0/8   }, {  "f2", 1.0/4   }, {"f1.4", 1.0/2   }
};


FILTER_DATA filter_data[NB_FILTERS] = {
	{ 400,  3.383545,  0.300,  1.29417,  0,  0,  -1 },
	{ 410,  5.906144,  0.310,  1.01786,  0,  1,   0 },
	{ 420,  3.619656,  0.318,  1.06500,  0,  0,   0 },
	{ 430,  3.584464,  0.327,  0.928017, 0,  0,  -1 },
	{ 440,  4.605772,  0.335,  0.955385, 0,  0,  -1 },
	{ 450,  2.728050,  0.341,  0.937748, 0,  0,   0 },
	{ 460,  3.243613,  0.351,  0.948348, 0, -1,   0 },
	{ 470,  4.003923,  0.360,  0.995810, 0,  0,   0 },
	{ 480,  3.353072,  0.370,  0.977000, 0,  0,   0 },
	{ 490,  4.329110,  0.378,  0.971591, 0,  0,   0 },
	{ 500,  3.583710,  0.387,  0.909336, 0,  1,   0 },
	{ 510,  5.362405,  0.395,  0.925414, 0,  0,   0 },
	{ 520,  5.268009,  0.402,  0.924191, 0,  0,   0 },
	{ 530,  5.855608,  0.408,  0.925121, 0,  0,   0 },
	{ 540,  6.127041,  0.415,  0.905873, 0,  0,   0 },
	{ 550,  5.328577,  0.423,  0.938406, 0,  0,   0 },
	{ 560,  5.586354,  0.430,  0.945251, 0,  0,  -2 },
	{ 570,  6.290892,  0.435,  0.944011, 0,  0,   0 },
	{ 580,  5.386822,  0.441,  0.881406, 0,  0,   0 },
	{ 590,  5.347115,  0.446,  0.933766, 0,  0,   1 },
	{ 600,  5.786929,  0.452,  0.833333, 0,  0,   0 },
	{ 610,  5.771357,  0.455,  0.788326, 0,  0,   0 },
	{ 620,  6.800541,  0.458,  0.740808, 0,  0,  -1 },
	{ 630,  6.213961,  0.460,  0.737011, 0,  0,   0 },
	{ 640,  5.434410,  0.461,  0.711362, 0,  0,   0 },
	{ 650,  6.468936,  0.463,  0.699933, 0,  0,   0 },
	{ 660,  7.281483,  0.465,  0.760027, 0,  0,   0 },
	{ 670,  7.121005,  0.466,  0.727157, 0, -2,  -2 },
	{ 680,  6.734717,  0.467,  0.684399, 0,  0,   0 },
	{ 690,  7.022411,  0.468,  0.763331, 0,  0,  -1 },
	{ 700,  7.630578,  0.469,  0.773630, 0,  0,   0 }
};

static float *lut_image=NULL;

read_lut_image( filename )
char *filename;
{
 ulong userdata=0, area=LUT_XS*LUT_YS*sizeof(float);
 short type=0, xs,ys;
 FILE *fpin=NULL;
  if( NULL == (fpin = fopen( filename, "r" )))
  { fprintf( stderr,"file: %s\t", filename ); error("can't read lut_file");}
  readTdata_imp_hdr( fileno(fpin), &type, &userdata, &xs, &ys );
  if( type != IMP3 ) error( "lut_image not IMP3"  );
  if( xs!=LUT_XS || ys!=LUT_YS )  error("lut_image wrong size");
  if( lut_image == NULL )
  {
    if( NULL == (lut_image = (float *) malloc( area )))
      error("not enough memory to make lut_image" );
  }
  read_bytes( fileno(fpin), lut_image, area );
  fclose( fpin );
}

float get_lut( grey, it )
int grey, it;
{
 ulong lut_offs;
  if( lut_image == NULL ) return( 0 );
  lut_offs = (LUT_YS-1-grey)*LUT_XS + it;
  return( lut_image[ lut_offs ] );
}

#define MAX_STR 256
#define SAME_STR( A, B ) ( 0 == strncmp( A, B, strlen(A) ) )

set_dra_flags( rootname, register_flag, old_590, grey_x, grey_y )
char *register_flag, *old_590;
int *grey_x, *grey_y;
{
 char *name, str[MAX_STR];
  name = (char *) tail( rootname ); /* tail in io.c */
  *register_flag = match_line_in_file( REGISTER_FILE, name, str );
  *old_590 = match_line_in_file( OLD590_FILE, name, str );
  if( match_line_in_file( GREYCARD_FILE, name, str ) )
  { sscanf( str, " %*s %d %d", grey_x, grey_y ); }
  else
  { *grey_x = *grey_y = -1; }
}

match_line_in_file( filename, name, str )
char *filename, *name, *str;
{
 FILE *fpin;
  if( NULL == (fpin = fopen( filename, "r" )))
  { fprintf( stderr,"can't open file: %s\t", filename ); return(0); }
  for( ; ; )
  {
     if( NULL == fgets( str, MAX_STR, fpin )) return(0); /* EOF and not found */
     if( SAME_STR( name, str ) ) return( 1 );  /* matched */
  }
}
